<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://docular.net
 * @since      1.0.0
 *
 * @package    Docular
 * @subpackage Docular/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Docular
 * @subpackage Docular/admin
 * @author     Your Name <michael@docular.net>
 */
class Docular_Admin {
	/**
	 * Contains any error returned from the API server.
	 *
	 * @since    1.0.1
	 * @access   private
	 * @var      string    $api_error    The error string
	 */
	public $api_error;



	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $docular    The ID of this plugin.
	 */
	private $docular;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The URI that points to the API. Enables us to globally control API version.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $api_uri    The URI that points to the API.
	 */
	private $api_uri;


	/**
	 * When obfuscating Client Secret and Domain Keys, we replace the first 24 characters with stars
	 *
	 * @since    1.1.0
	 * @access   protected
	 * @var      string    $stars    24 asterisk characters.
	 */
	protected $stars = "**************************";


	/**
	 * THIS VARIABLE SHOULD BE AN EMPTY ARRAY unless the testing server is being used in which case
	 * as it doesn't have an SSL certificate in which case use:
	 * private $SSL_context_option = array("ssl"=>array("verify_peer"=>false,"verify_peer_name"=>false,),);
	 *
	 * @since    1.0.0
	 * @param      string    $docular       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	 private $SSL_context_option;  


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $docular       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $docular, $version ) {

		$this->docular = $docular;
		$this->version = $version;
		$this->api_uri = "https://docular.net/api/v2/"; // must contain trailing slash!

		$this->SSL_context_option = array();  

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Docular_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Docular_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->docular, plugin_dir_url( __FILE__ ) . 'css/docular-admin.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'Font_Awesome', 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css' );





	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Docular_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Docular_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_script( $this->docular, plugin_dir_url( __FILE__ ) . 'js/docular-admin.js', array( 'jquery' ), $this->version, false );
	}




	/**
	 * Register the admin menu.
	 *
	 * @since    1.0.0
	 */
	public function docular_admin_menu() {
		add_menu_page( 'Docular - Privacy Policies, Terms &amp; Conditions', 'Docular', 'manage_options', 'docular-admin-menu', array( $this, 'docular_admin_settings' ), 'none');
	}


	/**
	 * Get admin page partial
	 *
	 * @since    1.0.0
	 */
	public function docular_admin_settings() {
		if ( !current_user_can( 'manage_options' ) )  {
			wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
		}
		include( plugin_dir_path( __FILE__ ) . 'partials/docular-admin-page-settings.php' );
	} 





	/**
	 * Hook for WP Ajax to add a API Client Secret
	 *
	 * @since    1.1.0
	 */
	function docular_ajax_add_clientsecret_action() {
		global $wpdb; // this is how you get access to the database
		$clientsecret = esc_attr($_POST["clientsecret"]);   
		$out = array();	
		if(strlen($clientsecret) == 32) {
			if(update_option("docular_api_clientsecret", $clientsecret)) {
				$hiddenclientsecret = $this->stars.substr($clientsecret, -6);
				$newhtml = "<div id=\"docular-display-clientsecret\">" . $hiddenclientsecret . "</div><button class=\"docular-admin-button docular-admin-button-light\" id=\"remove-clientsecret\">";
				$newhtml .= __("Remove Secret", 'docular');
				$newhtml .= "</button>";
				$out['success'] = $newhtml;
			}
			else {
				$out['error'] = __("Sorry, we were unable to save your Client Secret.", 'docular');
			}	
		}
		else {
			$out['error'] = __("Sorry, you didn't enter a valid Client Secret.", 'docular');
		}
		echo json_encode($out);
		wp_die(); // this is required to terminate immediately and return a proper response
	}
	
	
	/**
	 * Hook for WP Ajax to remove a API Client Secret
	 *
	 * @since    1.1.0
	 */
	function docular_ajax_remove_clientsecret_action() {
		$out = array();	
		if(!get_option('docular_api_domain_key')) {
			//Remove all docular options starting with docular- or docular_
			foreach (wp_load_alloptions() as $option => $value) {
				if(strpos( $option, 'docular-' ) === 0 || strpos( $option, 'docular_' ) === 0) {
					delete_option( $option );
				}
			}
		}
		else {
			delete_option('docular_api_clientsecret');
		}
		$newhtml = "<div class=\"docular-input-group\">";
		$newhtml .= "<input type=\"text\" id=\"api-clientsecret\" class=\"docular-admin-input\" placeholder=\"";
		$newhtml .= __("Your Docular API Client Secret");
		$newhtml .= "\" autocomplete=\"off\"><button class=\"docular-admin-button\" id=\"add-clientsecret\">";
		$newhtml .= __("Add Secret", 'docular');
		$newhtml .= "</button></div>";

		$out['success'] = $newhtml;
	
		echo json_encode($out);
		wp_die(); // this is required to terminate immediately and return a proper response
	}
	
	
	
	
	
	/**
	 * Hook for WP Ajax to add a API Domain Key
	 *
	 * @since    1.0.0
	 */
	function docular_ajax_add_domainkey_action() {
		global $wpdb; // this is how you get access to the database
		$domainkey = esc_attr($_POST["domainkey"]);   
		$out = array();	
		if(strlen($domainkey) == 32) {
			if(update_option("docular_api_domain_key", $domainkey)) {
				$hiddendomainkey = $this->stars.substr($domainkey, -6);
				$newhtml = "<div id=\"docular-display-domain-key\">" . $hiddendomainkey . "</div><button class=\"docular-admin-button docular-admin-button-light\" id=\"remove-domain-key\">";
				$newhtml .= __("Remove Key", 'docular');
				$newhtml .= "</button>";
				$out['success'] = $newhtml;
			}
			else {
				$out['error'] = __("Sorry, we were unable to save your Domain Key.", 'docular');
			}	
		}
		else {
			$out['error'] = __("Sorry, you didn't enter a valid Domain Key.", 'docular');
		}
		echo json_encode($out);
		wp_die(); // this is required to terminate immediately and return a proper response
	}
	
	
	/**
	 * Hook for WP Ajax to remove a API Domain Key
	 *
	 * @since    1.0.0
	 */
	function docular_ajax_remove_domainkey_action() {
		$out = array();	

		//Remove all docular options starting with docular- or docular_
		foreach (wp_load_alloptions() as $option => $value) {
			if(strpos( $option, 'docular-' ) === 0 || strpos( $option, 'docular_' ) === 0) {
				delete_option( $option );
			}
		}

		if(!get_option('docular_api_clientsecret')) {
			//Remove all docular options starting with docular- or docular_
			foreach (wp_load_alloptions() as $option => $value) {
				if(strpos( $option, 'docular-' ) === 0 || strpos( $option, 'docular_' ) === 0) {
					delete_option( $option );
				}
			}
		}
		else {
			delete_option('docular_api_domain_key');
		}

		$newhtml = "<div class=\"docular-input-group\">";
		$newhtml .= "<input type=\"text\" id=\"api-domain-key\" class=\"docular-admin-input\" placeholder=\"";
		$newhtml .= __("Your Docular API Domain Key");
		$newhtml .= "\" autocomplete=\"off\"><button class=\"docular-admin-button\" id=\"add-domain-key\">";
		$newhtml .= __("Add Key", 'docular');
		$newhtml .= "</button></div>";

		$out['success'] = $newhtml;
	
		echo json_encode($out);
		wp_die(); // this is required to terminate immediately and return a proper response
	}
	
	
	/**
	 * Hook for WP Ajax to retreive (for first time or to refresh) a Docular instance, and save as an option
	 *
	 * @since    1.0.0
	 */
	function docular_ajax_freshen_action() {
		global $wpdb; // this is how you get access to the database
		$documentkey = $_POST["documentkey"];
	
		if(strlen($documentkey) == 16) {
	
			$domainkey = get_option("docular_api_domain_key");
			$clientsecret = get_option("docular_api_clientsecret");
			$current = json_decode(get_option("docular-".$documentkey), true);
						
			$option = array(
				'documentkey' 	=> $documentkey,
				'modified' 	=> file_get_contents($this->api_uri . "instance/modified/" . $clientsecret . "/" . $domainkey . "/" . $documentkey, false, stream_context_create($this->SSL_context_option)),
				'css' 		=> file_get_contents($this->api_uri . "instance/css-min/". $clientsecret . "/" . $domainkey . "/" . $documentkey, false, stream_context_create($this->SSL_context_option)),
				'html' 		=> file_get_contents($this->api_uri . "instance/html/". $clientsecret . "/" . $domainkey . "/" . $documentkey, false, stream_context_create($this->SSL_context_option))
			);
	
			if(update_option("docular-".$documentkey, json_encode($option))) {
				$succ['freshness'] = "<i class=\"fa fa-leaf\"></i>&nbsp;" . __('Fresh', 'docular');
				$succ['shortcode'] = "[docular documentkey=\"" . $documentkey . "\"]";
				
				$out['success'] = $succ;
			}
			else {
				$out['error'] = __("Sorry, there was an error updating your document, please try later");			
			}
		}
		else {
			$out['error'] = __("Sorry, there was an error updating your document, please try later");		
		}	
	
	
		echo json_encode($out);
		wp_die(); // this is required to terminate immediately and return a proper response
	}
	
	/**
	 * Get a list (array/json/object) of all docular instances linked to this domain for the supplied Domain Key
	 *
	 * @since    1.0.0
	 */	
	function docular_get_instances($type='array') {

		global $wpdb; // this is how you get access to the database
		$clientsecret = get_option("docular_api_clientsecret");
		$domainkey = get_option("docular_api_domain_key");
	
		if($clientsecret && $domainkey) {
			$instances = @file_get_contents($this->api_uri . "instances/list/". $clientsecret . "/" . $domainkey);				

			if(isset($http_response_header)) {			
				foreach($http_response_header as $response) {
					if(strpos($response, "Docular-Error") !== false) {						
						$this->api_error = substr($response, 14);
					}
				}
			}

			if(strlen($instances) > 1) {
				if($type == 'json') $inst = $instances;
				elseif($type == 'object') $inst = json_decode($instances);
				else $inst = json_decode($instances, true);		

				$keys = array();
				foreach($inst as $i) {
					$keys[] = $i['documentkey'];
				}
				
				$all_options = wp_load_alloptions();
				$my_options  = array();
				 
				foreach ($all_options as $name => $value) {
					if (strpos($name, 'docular-') !== false) {
						$docuk = str_replace("docular-", "", $name);
						if(!in_array($docuk, $keys)) {
							delete_option($name);
						}
					}
				}

				return $inst;
			}
			else return null;
		}
		else return false;
	
	}

	/**
	 * Returns a table containing all docular instances linked to this domain for the supplied Domain Key
	 *
	 * @since    1.0.0
	 */	
	function docular_list_instances() {

		$instances = $this->docular_get_instances();

		$out = "";

		if(isset($this->api_error)) {
			$out .= "<div id=\"docularerrorblock\" class=\"docular-alert docular-error\">".$this->api_error."</div>";			
		}
		
		$out .= "<h2>" . __("Your Docular Documents", 'docular') . "</h2>";

		if($instances) {
		
			$out .= "<table class=\"widefat\" cellspacing=\"0\">";
				$out .= "<thead>";
					$out .= "<tr>";
						$out .= "<th>" . __('Document title', 'docular') . "</th>";
						$out .= "<th>" . __('Last modified', 'docular') . "</th>";
						$out .= "<th>" . __('Document freshness', 'docular') . "</th>";
						$out .= "<th>" . __('Shortcode', 'docular') . "</th>";
					$out .= "</tr>";
				$out .= "</thead>";
				$out .= "<tfoot>";
					$out .= "<tr>";
						$out .= "<th>" . __('Document title', 'docular') . "</th>";
						$out .= "<th>" . __('Last modified', 'docular') . "</th>";
						$out .= "<th>" . __('Document freshness', 'docular') . "</th>";
						$out .= "<th>" . __('Shortcode', 'docular') . "</th>";
					$out .= "</tr>";
				$out .= "</tfoot>";
				$out .= "<tbody>";
					$i=0;
					foreach($instances as $instance) {

						$stored = json_decode(get_option("docular-".$instance['documentkey']),true);

						//$altclass = ($i%2 == 0) ? 'alternate' : '';
						if($i%2 == 0) $altclass = 'alternate';
						else $altclass = '';


						$out .= "<tr class=\"" . $altclass . "\">";
							$out .= "<td class=\"docular-table-title\">" . $instance['title'] . "</td>";
							$out .= "<td>" . $instance['modified'] . "</td>";
							$out .= "<td class=\"docular-freshness\">";
								if(isset($stored)) {
									if($stored['modified'] < $instance['modified']) {
										$out .= "<button class=\"docular-admin-button docular-admin-button-light docular-refresh-button\" data-documentkey=\"".$instance['documentkey']."\"><i class=\"fa fa-refresh\"></i>&nbsp;".__('Refresh', 'docular')."</button>";
									}
									else {
										$out .= "<i class=\"fa fa-leaf\"></i>&nbsp;";
										$out .= __('Fresh', 'docular');
									}
								}
							$out .= "</td>";
							$out .= "<td class=\"docular-shortcode\">";
																
								if(isset($stored['documentkey']) && $stored['documentkey'] == $instance['documentkey']) {
									$out .= "[docular documentkey=\"" . $instance['documentkey'] . "\"]";
								}
								else {
									$out .= "<button class=\"docular-admin-button docular-admin-button-light docular-refresh-button\" data-documentkey=\"" . $instance['documentkey'] . "\"><i class=\"fa fa-plus\"></i>&nbsp;".__('Get shortcode', 'docular')."</button>";
								}
							$out .= "</td>";
						$out .= "</tr>";
						$i++;
					}
				$out .= "</tbody>";
			$out .= "</table>";
		} 
		else {
			$out .= "<p><strong>".__("You don't have any documents yet", 'docular')."</strong></p>";
			$out .= "<p>".__('Head over to <a href="https://docular.net">Docular</a> and browse the library of legal documents. Give your document an API Document Key (<a href="https://docular.net/pages/api">find out how to do that</a>) and your document will appear here.', 'docular')."</p>";
		}
		return $out;
	}

	
	/**
	 * Hook for WP Ajax to get documents table via docular_list_instances()
	 *
	 * @since    1.0.0
	 */
	function docular_ajax_list_instances_action() {
		echo $this->docular_list_instances();
		wp_die(); // this is required to terminate immediately and return a proper response
	}



	/**
	 * Display the uri and/or ip from which the plugin connects to the api.
	 *
	 * @since    1.0.0
	 */
	public function show_connections() {

		$uri = file_get_contents($this->api_uri . "hostinfo/domain");
		$ip = file_get_contents($this->api_uri . "hostinfo/ip");

		if($uri) $out .= sprintf(__("When you create your API Domain Key, use either <span class=\"docular-code\">%s</span> or <span class=\"docular-code\">%s</span> in the 'domain'."), $uri, $ip)."</p>";
		else $out .= sprintf(__("When you create your API Domain Key, use <span class=\"docular-code\">%s</span> in the 'domain'."), $ip)."</p>";

		return $out;

	}






}
